@@ -1,17 +1,16 @@
 import copy
 import json
+import builtins
 
 """Struct class to handle complex JSON-style data structures"""
 
 
 class Struct:
-    def __init__(self, data: "JSON style object"):
-        """
-
-        :param data: dict or list object
-        """
-        if type(data) != dict and type(data) != list:
-            raise TypeError("Argument should be a dict or a list, not a {0}".format(str(type(data))))
+    def __init__(self, data=None):
+        if data is None:
+            self.data = {}
+        elif type(data) != dict and type(data) != list:
+            self.data = deserialize(data)
         else:
             self.data = data
 
@@ -39,12 +38,18 @@ class Struct:
 
     def __setitem__(self, path, value):
         """self.__setitem__((path, to, value), value) --> self[path, to, value]=value"""
-        if type(path) == str:
+        if type(path) == tuple:
+            path = list(path)
+        elif type(path) != list:
             path = [path]
         modifyStruct(self.data, list(path) + [value])
 
     def __delitem__(self, path):
-        self[path] = None
+        if type(path) == tuple:
+            path = list(path)
+        elif type(path) != list:
+            path = [path]
+        delItem(self.data, path)
 
     def __contains__(self, key):
         """Returns (key in self)"""
@@ -52,7 +57,9 @@ class Struct:
 
     def __getitem__(self, path):
         """self.__getitem__(path) --> self[path]"""
-        if type(path) == str:
+        if type(path) == tuple:
+            path = list(path)
+        elif type(path) != list:
             path = [path]
         return getItem(self.data, list(path))
 
@@ -133,18 +140,54 @@ def getItem(data: 'JSON style object', path):
         return data
     return getItem(data[path[0]], path[1:])
 
+def delItem(data: "JSON style object", path):
+    # print(len(path)
+    if not path:
+        raise KeyError("Invalid path")
+    elif len(path) == 1:
+        del data[path[0]]
+    else:
+        delItem(data[path[0]], path[1:])
+
+def deserialize(obj):
+    data = {}
+    defaultTypes = [
+        int,
+        str,
+        float,
+        bool,
+        list,
+        dict,
+        complex,
+        tuple
+    ]
+    excludedTypes = [
+        "function",
+        "builtins.staticmethod",
+        "classmethod",
+        "builtin_function_or_method",
+        "method-wrapper",
+        "NoneType"
+    ]
+    attrs = dir(obj)
+    attrs.remove('__class__')
+    attrs.remove('__doc__')
+    for attr in attrs:
+        v = obj.__getattribute__(attr)
+        if type(v) in defaultTypes:
+            data[attr] = v
+        elif type(v).__name__ not in excludedTypes:
+            data[attr] = str(v)
+    return data
+
 
 def modifyStruct(data: 'JSON style object', path):
     if len(path) == 2:
         data[path[0]] = path[1]
-        if path[1] is None:
-            del data[path[0]]
         return data
     if type(data) == dict:
         data.setdefault(path[0], {})
     data[path[0]] = modifyStruct(data[path[0]], path[1:])
-    if not data[path[0]]:
-        del data[path[0]]
     return data
 
 
